package com.brilliance.isc.funds.fttfcn.service;

import com.brilliance.isc.bo.Bch;
import com.brilliance.isc.bo.Ftd;
import com.brilliance.isc.bo.Pta;
import com.brilliance.isc.bo.model.Ftdgrp;
import com.brilliance.isc.bo.model.Fttp;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.component.SptDataComponent;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.liaall.service.LiaallService;
import com.brilliance.isc.common.setglg.vo.SetglgVo;
import com.brilliance.isc.common.setmod.service.SetmodService;
import com.brilliance.isc.common.setmod.vo.SetmodVo;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.bo.funds.FttfcnStoreBo;
import com.brilliance.isc.funds.common.component.FtToolComponent;
import com.brilliance.isc.funds.fttfcn.check.FttfcnValidator;
import com.brilliance.isc.funds.transfer.transaction.FttfcnStructMapper;
import com.brilliance.isc.funds.vo.funds.FttfcnStoreVo;
import com.brilliance.isc.funds.vo.funds.FttfcnStoreVo;
import com.brilliance.isc.mda.dao.ActMapper;
import com.brilliance.isc.mda.dao.PtyratMapper;
import com.brilliance.isc.vo.funds.FtdBizInfoVo;
import com.brilliance.mda.runtime.mda.util.*;
import com.fasterxml.jackson.annotation.JsonIgnore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.brilliance.mda.runtime.mda.Constants.*;

/**
 * @program: isc-funds
 * @ClassName: FttfcnTransactionServiceImpl
 * @description: 结售汇平盘确认 fttfcn 业务处理
 * @author: xiaotong
 * @create: 2024-10-11 15:20
 */

@Service("fttfcn.transaction")
public class FttfcnTransactionServiceImpl extends AbstractTransactionService<FttfcnStoreBo> {
    @Resource
    private FttfcnValidator fttfcnValidator;
//    @Autowired
//    private FttfcnService fttfcnService;
    @Resource
    private FtToolComponent ftToolComponent;
    @Resource
    private TrnmodService trnmodService;
    @Resource
    private XrtmodService xrtmodService;
    @Resource
    private TransactionHelpService transactionHelpService;
    @Resource
    private CbsmodService cbsmodService;
    @Resource
    private LiaallService liaallService;
    @Resource
    private SettleContext settleContext;
    @Autowired
    private PtaCacheService ptaCacheService;

    @Resource
    private BchService bchService;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private PtyratMapper ptyratMapper;

    @Resource
    private ActMapper actMapper;

    @Resource
    private SetmodService setmodService;

    @Autowired
    private FttfcnStructMapper fttfcnStructMapper;


    @Autowired
    SptDataComponent sptDataComponent;

    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDIA_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");

//        serviceRegisterMaps.put(CommonContants.UMDMOD_SERVICE, "N");
//        serviceRegisterMaps.put(CommonContants.LIMMOD_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_RMB_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_SZSB_BOPREM_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_CFABOP_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.FANXIQIAN_SERVICE, "Y");
    }

    @Override
    protected void txnSave(FttfcnStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    @Override
    protected Map<String, String> txnCheck(FttfcnStoreBo transactionVo) {
        return fttfcnValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FttfcnStoreBo transactionVo) {
        FtdBizInfoVo bizInfoVo = new FtdBizInfoVo();
        bizInfoVo.setObjtyp("FTD");
        Ftd rec = transactionVo.getFtdgrp().getRec();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setOwnref(rec.getOwnref());
        bizInfoVo.setFttyp(rec.getFttyp());
        bizInfoVo.setCanamt(transactionVo.getCanamt());
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FttfcnStoreBo transactionVo) {
        return null;
    }

    @Override
    public FttfcnStoreBo init(FttfcnStoreBo fttfcnStoreBo) {
        fttfcnStoreBo.reset();

        String ftdinr = fttfcnStoreBo.getFtdgrp().getRec().getInr();
        if (sptDataComponent.needMergeSptData(fttfcnStoreBo)) {
            ftdinr = fttfcnStoreBo.getSpt().getObjinr();
        }

        Ftdgrp ftdgrpVo = ftToolComponent.getFtdgrpByInr(ftdinr);
        fttfcnStoreBo.setFtdgrp(ftdgrpVo);

        
        init950(fttfcnStoreBo);
        init1000(fttfcnStoreBo);

        if( MdaUtils.isEmpty(fttfcnStoreBo.getFtdgrp().getRec().getOpndat()) )
        {
            fttfcnStoreBo.getFtdgrp().getRec().setOpndat(  Dates.today());
        }
        if (  MdaUtils.isEmpty(fttfcnStoreBo.getCanamt())){
            fttfcnStoreBo.setCanamt(ftdgrpVo.getCbs().getOpn2().getAmt());
        }

        return fttfcnStoreBo;
    }

    private void init950(FttfcnStoreBo fttfcnStoreBo) {
    }

    private void init1000(FttfcnStoreBo bo) {

    }

    @Override
    public void beforeSave(FttfcnStoreBo transactionVo) {
        transactionVo.reset();
        Ftdgrp ftdgrp = transactionVo.getFtdgrp();

        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(ftdgrp.getRec());
        transactionServiceSet.setBlk(ftdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        ftToolComponent.collectCbsMap(transactionServiceSet, "", ftdgrp.getCbs(), "max", "ins", "opn1", "opn2");
        ftToolComponent.collectPtsList(transactionServiceSet, ftdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FttfcnStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getFtdgrp().getRec().getInr());
    }

    public void ruleMtabutSavN500(FttfcnStoreBo bo) {
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();

        ftToolComponent.getref(bo.getFtdgrp());
        // 1. Create (rsp. update) Contract (incl. Texts)
        ftToolComponent.sav(bo.getFtdgrp());
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract ( trnmodVo);
        cbsSav(bo,trnmodVo);
        // save documents
        //TODO
        //记录116台账
        //transactionVo.getNosmod().savNostro();

    }

    public String getRef(FttfcnStoreVo fttfcnStorevo) {
        FttfcnStoreBo fttfcnStoreBo= fttfcnStructMapper.voTransferToBo(fttfcnStorevo);
        return getRef(fttfcnStoreBo);
    }

    public String getRef(FttfcnStoreBo fttfcnStoreBo) {
        fttfcnStoreBo.reset();
        PtsptaVo ptsptaVo = fttfcnStoreBo.getFtdgrp().getTro();

        if (ptsptaVo == null || StringUtils.isEmpty(ptsptaVo.getPtainr())) {
            SettleContext.getUserSession().setWrkbch(SettleContext.getUserSession().getBch());
        }else{
            Pta pta = ptaCacheService.getPtaByInr(ptsptaVo.getPtainr());
            Bch wrkbch = new Bch();
            wrkbch.setInr(pta.getBranchinr());
            wrkbch = bchService.selectByPrimaryKey(wrkbch);
            SettleContext.getUserSession().setWrkbch(wrkbch);
        }

        Ftdgrp ftdgrp = fttfcnStoreBo.getFtdgrp();
        ftToolComponent.getref(ftdgrp);
        return ftdgrp.getRec().getOwnref();
    }

    public void cbsSav(FttfcnStoreBo transactionVo, TrnmodVo trnmodVo) {
        String trninr = transactionHelpService.getTrninr();
        SetmodVo setmodVo = transactionHelpService.getTransactionStoreSet().getSetmodVo();

        cbsmodService.cbsTrnSetNew("TRN",trninr);
        trnmodService.cbsTrnStore("OPNAMT","AMT1",Dates.today(),setmodVo.getDoccur(),transactionVo.getCanamt().negate(),"Funds Transfer",transactionVo.getFtdgrp().getRec(),trninr);
        cbsmodService.cbsTrnCommit();


    }




    //@Default(target="mtabut.clsflg",order=5000)
//    public boolean defaultMtabutclsflgN5000(FttfcnStoreBo fttfcnStoreBo){
//        Ftdgrp ftdgrp = fttfcnStoreBo.getFtdgrp();
//        SetmodVo setmodVo = transactionHelpService.getTransactionStoreSet().getSetmodVo();
//        // Close Flag is defaulted with X, if the conditions for setting close date like in MTABUT\\SAV are fulfilled
//        boolean clsflg = true;
//        if(  ! MdaUtils.isEmpty(ftdgrp.getRec().getCnfdat()) )
//        {
//            if( MdaUtils.compareTo(ftdgrp.getCbs().getOpn1().getAmt(),0)>0 )
//            {
//                clsflg = false;
//            }
//            if( MdaUtils.compareTo(ftdgrp.getCbs().getOpn2().getAmt(),fttfcnStoreBo.getCanamt()) != 0 )
//            {
//                clsflg = false;
//            }
//            if( clsflg )
//            {
//                if( ! Systems.isModified(this.getMtabut(),"clsflg") )
//                {
//                    this.getMtabut().setclsflg("C");
//                }
//                Systems.setEnabled(this.getMtabut(),"clsflg");
//            }
//            else
//            {
//                Dynamic.disableAndClearField( MdaDriver.getDatafield(this.getMtabut(),"clsflg") );
//                this.getMtabut().setclsflg("O");
//            }
//        }
//        else
//        {
//            Dynamic.disableAndClearField( MdaDriver.getDatafield(this.getMtabut(),"clsflg") );
//            this.getMtabut().setclsflg("O");
//        }
//
//
//        return true;
//    }








}
