package com.brilliance.isc.funds.fxtlcm.service.impl;

import com.brilliance.isc.bo.Bch;
import com.brilliance.isc.bo.Fxd;
import com.brilliance.isc.bo.Pta;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.gmgf.service.GmgfService;
import com.brilliance.isc.common.liaall.service.LiaallService;
import com.brilliance.isc.common.setmod.service.SetmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.funds.bo.funds.FttfcmStoreBo;
import com.brilliance.isc.funds.bo.funds.FxtlcmStoreBo;
import com.brilliance.isc.funds.common.component.FxToolComponent;
import com.brilliance.isc.funds.fxtlcm.check.FxtlcmValidator;
import com.brilliance.isc.funds.transfer.transaction.FxtlcmStructMapper;
import com.brilliance.isc.funds.vo.funds.FttfcmStoreVo;
import com.brilliance.isc.funds.vo.funds.FxtfcmStoreVo;
import com.brilliance.isc.funds.vo.funds.FxtlcmStoreVo;
import com.brilliance.isc.mda.dao.ActMapper;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: FxtlcmTransactionServiceImpl
 * @description: 结售汇平盘登记Fxtlcm 业务处理
 * @author: Lenovo
 * @create: 2024-10-14 11:53
 */
@Service("fxtlcm.transaction")
public class FxtlcmTransactionServiceImpl extends AbstractTransactionService<FxtlcmStoreBo> {
    @Resource
    private ActMapper actMapper;
    @Resource
    private FxtlcmValidator fxtlcmValidator;
    @Resource
    private FxToolComponent fxToolComponent;
    @Resource
    private TrnmodService trnmodService;
    @Resource
    private TransactionHelpService transactionHelpService;
    @Resource
    private SettleContext settleContext;
    @Autowired
    private PtaCacheService ptaCacheService;
    @Resource
    private BchService bchService;

    @Resource
    private LiaallService liaallService;
    @Resource
    private CbsmodService cbsmodService;

    @Autowired
    private FxtlcmStructMapper fxtlcmStructMapper;


    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDIA_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.UMDMOD_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.LIMMOD_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.BOPSET_RMB_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_SZSB_BOPREM_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_CFABOP_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.FANXIQIAN_SERVICE, "Y");
    }

    @Override
    protected void txnSave(FxtlcmStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    @Override
    protected Map<String, String> txnCheck(FxtlcmStoreBo transactionVo) {
        return fxtlcmValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FxtlcmStoreBo transactionVo) {
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        Fxd rec = transactionVo.getFxdgrp().getRec();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setOwnref(rec.getOwnref());
        bizInfoVo.setFxtyp(rec.getFxtyp());
        bizInfoVo.setAcc1(transactionVo.getAcc1());
        bizInfoVo.setAcc2(transactionVo.getAcc2());
        bizInfoVo.setDsp(rec.getDsp());
        bizInfoVo.setDsp2(rec.getDsp2());
        bizInfoVo.setTrdint(rec.getTrdint());
        bizInfoVo.setTrdout(rec.getTrdout());
        bizInfoVo.setTrnman(rec.getTrnman());
        bizInfoVo.setRat(rec.getRat());
        bizInfoVo.setValdat(rec.getValdat());
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FxtlcmStoreBo transactionVo) {
        return null;
    }

    @Override
    public FxtlcmStoreBo init(FxtlcmStoreBo fxtlcmStoreBo) {
        fxtlcmStoreBo.reset();
        initFxtlcm(fxtlcmStoreBo);
        //fxtlopService.defaultFxtlop(fxtlopStoreBo);
        return fxtlcmStoreBo;
    }

    public void initFxtlcm(FxtlcmStoreBo fxtlcmStoreBo){
        Fxdgrp fxdgrp = fxtlcmStoreBo.getFxdgrp();
        String fxdinr = fxdgrp.getRec().getInr();

        fxdgrp = fxToolComponent.getFxdgrpByInr(fxtlcmStoreBo.getFxdgrp().getRec().getInr());

        String extkey = SettleContext.getUserSession().getUsr().getExtkey();

        if (MdaUtils.isEmpty(fxdgrp.getRec().getOwnusr())){
            fxdgrp.getRec().setOwnusr(extkey);
        }
        if (MdaUtils.isEmpty(fxdgrp.getRec().getUsr())){
            fxdgrp.getRec().setUsr(extkey);
        }
        fxtlcmStoreBo.setFxdgrp(fxdgrp);
    }



    @Override
    public void beforeSave(FxtlcmStoreBo transactionVo) {
        transactionVo.reset();
        Fxdgrp fxdgrp = transactionVo.getFxdgrp();
        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(fxdgrp.getRec());
        transactionServiceSet.setBlk(fxdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        fxToolComponent.collectCbsMap(transactionServiceSet, "", fxdgrp.getCbs(), "max", "nom1", "opn1", "opn2");
        fxToolComponent.collectPtsList(transactionServiceSet, fxdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FxtlcmStoreBo transactionVo) {
    }

    public void ruleMtabutSavN500(FxtlcmStoreBo transactionVo){
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();
        Fxdgrp fxdgrp=transactionVo.getFxdgrp();

        fxToolComponent.getref(transactionVo.getFxdgrp());

        //# 1. Create (rsp. update) Contract (incl. Texts)
        fxdgrp.getRec().setCnfdat(Dates.today());

        if (fxdgrp.getCbs().getOpn2().getAmt().compareTo(BigDecimal.ZERO)==0){
            fxdgrp.getRec().setClsdat(Dates.today());
        }

        fxdgrp.setAcc(transactionVo.getAplacc());

        // 1. Create (rsp. update) Contract (incl. Texts)
        fxToolComponent.sav(transactionVo.getFxdgrp(), BigDecimal.ZERO);
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract ( trnmodVo);
        cbsSav(transactionVo, trnmodVo);
        // save documents
//        #记录116台账
//        NOSMOD.savNostro
    }

    public void cbsSav(FxtlcmStoreBo transactionVo, TrnmodVo trnmodVo) {
        cbsmodService.cbsTrnSetNew("TRN",trnmodVo.getTrn().getInr());
//        //#cgl
//        //平盘登记后产生4条CBB,MAX,NOM,OPN,OPN2
//        //后续两个交易分别消OPN和OPN2
//        String trninr = transactionHelpService.getTrninr();
//        trnmodService.cbsTrnStore("MAXAMT","AMT1", Dates.today(),transactionVo.getFxdgrp().getCbs().getMax().getCur(),transactionVo.getFxdgrp().getCbs().getMax().getAmt(), MdaUtils.getI18NString("fxtlop","CT000000"),transactionVo.getFxdgrp().getRec(),trninr);
//        trnmodService.cbsTrnStore("NOMAMT","AMT1",Dates.today(),transactionVo.getFxdgrp().getCbs().getNom1().getCur(),transactionVo.getFxdgrp().getCbs().getNom1().getAmt(),MdaUtils.getI18NString("fxtlop","CT000001"),transactionVo.getFxdgrp().getRec(),trninr);
//        /**
//         if FXDGRP\\REC\\FXTYP.is ( "LB" ) or FXDGRP\\REC\\FXTYP.is ( "LS" ) then
//
//         **/
//        /**
//         if FXDGRP\\REC\\FXTYP.is ( "LB" ) then
//
//         **/
//        if( MdaUtils.compareTo(transactionVo.getFxdgrp().getRec().getFxtyp(),"LB")== 0 || MdaUtils.compareTo(transactionVo.getFxdgrp().getRec().getFxtyp(),"LS")== 0 )
//        {
//            trnmodService.cbsTrnStore("OPNAMT","AMT1",Dates.today(),transactionVo.getFxdgrp().getCbs().getMax().getCur(),transactionVo.getFxdgrp().getCbs().getMax().getAmt(),MdaUtils.getI18NString("fxtlop","CT000002"),transactionVo.getFxdgrp().getRec(),trninr);
//        }
//        if ( !MdaUtils.isEmpty( transactionHelpService.getTransactionStoreSet().getLiaall())) {
//            liaallService.storeAll(transactionHelpService.getTransactionStoreSet().getLiaall());
//        }
           // liaallService.storeAll(transactionHelpService.getTransactionStoreSet().getLiaall());

            cbsmodService.cbsTrnCommit();
    }

    public String getRef(FxtlcmStoreVo fxtlcmStoreVo) {
        FxtlcmStoreBo fxtlcmStoreBo= fxtlcmStructMapper.voTransferToBo(fxtlcmStoreVo);
        return getRef(fxtlcmStoreBo);
    }

    public String getRef(FxtlcmStoreBo fxtlcmStoreBo) {
        fxtlcmStoreBo.reset();
        PtsptaVo ptsptaVo = fxtlcmStoreBo.getFxdgrp().getApl();
        if (ptsptaVo == null || StringUtils.isEmpty(ptsptaVo.getPtainr())) {
            return NULLSTR;
        }
        Pta pta = ptaCacheService.getPtaByInr(ptsptaVo.getPtainr());
        Bch wrkbch = new Bch();
        wrkbch.setInr(pta.getBranchinr());
        wrkbch = bchService.selectByPrimaryKey(wrkbch);
        if (Objects.isNull(wrkbch)) {
            return NULLSTR;
        }
        SettleContext.getUserSession().setWrkbch(wrkbch);
        Fxdgrp fxdgrp = fxtlcmStoreBo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        return fxdgrp.getRec().getOwnref();
    }

    public List<String> defaultAcc2N1000(FxtlcmStoreVo vo){
        vo.reset();
        FxtlcmStoreBo bo= fxtlcmStructMapper.voTransferToBo(vo);
        Fxd rec=bo.getFxdgrp().getRec();

        List<String> actLst=new ArrayList<>();
        if( ! MdaUtils.isEmpty(vo.getFxdgrp().getCbs().getMax().getCur()) && ! MdaUtils.isEmpty(vo.getAplacc().getPts().getPtyinr()) )
        {
            String sql = "WHERE CUR = '" + vo.getFxdgrp().getCbs().getMax().getCur() + "' AND SERPTYINR = '" + vo.getAplacc().getPts().getPtyinr() + "' AND TYP = 'DB'";
            actLst = actMapper.getSerAccByConditionSql(sql);

            if( ! vo.isModified("acc2") && actLst.size()>0) {
                vo.setAcc2( actLst.get(0));
            }
        }
        else
        {
            vo.setAcc2(NULLSTR);
        }


        return actLst;
    }

    /**
     *   source:fxtfcm.0009.script
     */

    public  List<String> defaultAcc1N1000(FxtlcmStoreVo vo){
        vo.reset();
        FxtlcmStoreBo bo= fxtlcmStructMapper.voTransferToBo(vo);
        Fxd rec=bo.getFxdgrp().getRec();

        List<String> actLst=new ArrayList<>();
        if( ! MdaUtils.isEmpty(vo.getFxdgrp().getCbs().getMax().getCur()) && ! MdaUtils.isEmpty(vo.getFxdgrp().getAct().getPts().getPtyinr()) )
        {
            String sql = "WHERE CUR = '" + vo.getFxdgrp().getCbs().getMax().getCur() + "' AND SERPTYINR = '" + vo.getFxdgrp().getAct().getPts().getPtyinr() + "' AND TYP = 'DB'";
            actLst = actMapper.getSerAccByConditionSql(sql);

            if( ! vo.isModified("acc1") && actLst.size()>0) {
                vo.setAcc1( actLst.get(0));
            }
        }
        else
        {
            vo.setAcc1(NULLSTR);
        }


        return actLst;
    }
}
