package com.brilliance.isc.funds.fxtlcn.service;

import com.brilliance.isc.bo.Act;
import com.brilliance.isc.bo.Fxd;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.bo.Smh;
import com.brilliance.isc.bo.Pty;
import com.brilliance.isc.common.api.bch.service.BchService;
import com.brilliance.isc.common.cache.pta.PtaCacheService;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.liaall.service.LiaallService;
import com.brilliance.isc.common.setmod.service.SetmodService;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.funds.bo.funds.FxtlcnStoreBo;
import com.brilliance.isc.funds.common.component.FxToolComponent;
import com.brilliance.isc.funds.fxtlcn.check.FxtlcnValidator;
import com.brilliance.isc.funds.fxtlop.service.FxtpService;
import com.brilliance.isc.funds.vo.funds.FxtlcnStoreVo;
import com.brilliance.isc.mda.dao.PtyratMapper;
import com.brilliance.isc.mda.dao.SmhMapper;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.Decimals;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.brilliance.isc.mda.dao.ActMapper;
import com.brilliance.isc.mda.dao.PtyMapper;
import com.brilliance.isc.common.component.SptDataComponent;
import com.brilliance.isc.common.component.PtsptaToolComponent;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

/**
 * @program: isc-funds
 * @ClassName: FxtlcnTransactionServiceImpl
 * @description: 结售汇平盘确认 Fxtlcn 业务处理
 * @author: xiaotong
 * @create: 2024-10-11 15:20
 */

@Service("fxtlcn.transaction")
public class FxtlcnTransactionServiceImpl extends AbstractTransactionService<FxtlcnStoreBo> {
    @Resource
    private FxtlcnValidator fxtlcnValidator;
//    @Autowired
//    private FxtlcnService fxtlcnService;
    @Resource
    private FxToolComponent fxToolComponent;
    @Resource
    private TrnmodService trnmodService;
    @Resource
    private XrtmodService xrtmodService;
    @Resource
    private TransactionHelpService transactionHelpService;
    @Resource
    private CbsmodService cbsmodService;
    @Resource
    private LiaallService liaallService;
    @Resource
    private SettleContext settleContext;
    @Autowired
    private PtaCacheService ptaCacheService;

    @Resource
    private BchService bchService;

    @Resource
    private PtyratMapper ptyratMapper;

    @Resource
    private SmhMapper smhMapper;

    @Resource
    private ActMapper actMapper;

    @Resource
    private PtyMapper ptyMapper;

    @Resource
    private SetmodService setmodService;

    @Resource
    private FxtpService fxtpService;

    @Resource
    private SysmodService sysmodService;

    @Autowired
    SptDataComponent sptDataComponent;

    @Autowired
    PtsptaToolComponent ptsptaToolComponent;

    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDIA_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");

//        serviceRegisterMaps.put(CommonContants.UMDMOD_SERVICE, "N");
//        serviceRegisterMaps.put(CommonContants.LIMMOD_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_RMB_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_SZSB_BOPREM_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.BOPSET_CFABOP_SERVICE, "Y");
//        serviceRegisterMaps.put(CommonContants.FANXIQIAN_SERVICE, "Y");
    }

    @Override
    protected void txnSave(FxtlcnStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    @Override
    protected Map<String, String> txnCheck(FxtlcnStoreBo transactionVo) {
        return fxtlcnValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FxtlcnStoreBo bo) {
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        Fxd rec = bo.getFxdgrp().getRec();
        bizInfoVo.setObjinr(rec.getInr());
        bizInfoVo.setOwnref(rec.getOwnref());
        bizInfoVo.setFxtyp(rec.getFxtyp());
        bizInfoVo.setAcc1(bo.getAcc1());
        bizInfoVo.setAcc2(bo.getAcc2());
        bizInfoVo.setDsp(rec.getDsp());
        bizInfoVo.setDsp2(rec.getDsp2());
        bizInfoVo.setTrdint(rec.getTrdint());
        bizInfoVo.setTrdout(rec.getTrdout());
        bizInfoVo.setTrnman(rec.getTrnman());
        bizInfoVo.setRat(rec.getRat());
        bizInfoVo.setCancur(bo.getCancur());
        bizInfoVo.setCanamt(bo.getCanamt());

        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FxtlcnStoreBo transactionVo) {
        return null;
    }

    @Override
    public FxtlcnStoreBo init(FxtlcnStoreBo fxtlcnStoreBo) {
        fxtlcnStoreBo.reset();
        String fxdinr = fxtlcnStoreBo.getFxdgrp().getRec().getInr();
        if (sptDataComponent.needMergeSptData(fxtlcnStoreBo)) {
            fxdinr = fxtlcnStoreBo.getSpt().getObjinr();
        }

        Fxdgrp fxdgrpVo = fxToolComponent.getFxdgrpByInr(fxdinr);
        fxtlcnStoreBo.setFxdgrp(fxdgrpVo);

        init950(fxtlcnStoreBo);
        init1000(fxtlcnStoreBo);
        return fxtlcnStoreBo;
    }

    private void init950(FxtlcnStoreBo fxtlcnStoreBo) {
        Fxdgrp fxdgrp=fxtlcnStoreBo.getFxdgrp();

        if ( ! fxtlcnStoreBo.isModified("canamt")){
            fxtlcnStoreBo.setCanamt(fxdgrp.getCbs().getOpn2().getAmt());
        }
        if ( ! fxtlcnStoreBo.isModified("cancur")){
            fxtlcnStoreBo.setCancur(fxdgrp.getCbs().getOpn2().getCur());
        }



        if (  MdaUtils.isEmpty(fxdgrp.getRec().getOpndat())){
            fxdgrp.getRec().setOpndat(Dates.today());
        }


    }

    private void init1000(FxtlcnStoreBo fxtlcnStoreBo) {

        if( MdaUtils.isEmpty(fxtlcnStoreBo.getOldactinr()) )
        {
            if( ! MdaUtils.isEmpty(fxtlcnStoreBo.getFxdgrp().getAct().getPts().getPtyinr()) )
            {
                fxtlcnStoreBo.setOldactinr( fxtlcnStoreBo.getFxdgrp().getAct().getPts().getPtyinr());
            }
            else
            {
                fxtlcnStoreBo.setOldactinr( "NONE");
            }
        }

        String extKey = NULLSTR;
        String relCur = NULLSTR;
        BigDecimal relAmt = Decimals.ZERO_SCALE2;
        String f21 = NULLSTR;
        String inr = NULLSTR;
        if( sysmodService.sptIsIncLoaded(fxtlcnStoreBo.getSpt().getSta()) )
        {

            Smh smh=smhMapper.getSmhInfoBySptSmhinr(fxtlcnStoreBo.getSpt().getSmhinr());
            // 通过报文进来，读取报文发报行
            if (smh !=null){
                extKey = smh.getSndkey();
                relCur = smh.getRelcur();
                relAmt = smh.getRelamt();
                //todo
                //f21 =smh.getF21();
            }


            fxtlcnStoreBo.setRelcur( relCur);
            fxtlcnStoreBo.setRelamt(relAmt);
            fxtlcnStoreBo.setF21( f21);
            if( ! MdaUtils.isEmpty(extKey) )
            {
                if( Strings.len(extKey) == 12 )
                {
                    extKey = Strings.mid(extKey,1,8) + Strings.mid(extKey,10);
                }

                Pty pty=ptyMapper.selectByExtkey(extKey);

                inr = "";
                inr = pty.getInr();

                if( ! MdaUtils.isEmpty(inr) && MdaUtils.compareTo(fxtlcnStoreBo.getOldactinr(),inr) != 0 )
                {
                    fxtlcnStoreBo.getFxdgrp().setAct(ptsptaToolComponent.getPtsptaFromPtyINR(  inr ));
                }
            }
        }
        fxtlcnStoreBo.setAplacc(fxtlcnStoreBo.getFxdgrp().getAcc());
    }

    @Override
    public void beforeSave(FxtlcnStoreBo transactionVo) {
        transactionVo.reset();
        Fxdgrp fxdgrp = transactionVo.getFxdgrp();

        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(fxdgrp.getRec());
        transactionServiceSet.setBlk(fxdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        fxToolComponent.collectCbsMap(transactionServiceSet, "", fxdgrp.getCbs(), "max", "nom1", "opn1", "opn2");
        fxToolComponent.collectPtsList(transactionServiceSet, fxdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FxtlcnStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getFxdgrp().getRec().getInr());
    }

    public void ruleMtabutSavN500(FxtlcnStoreBo bo) {
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();

        fxToolComponent.getref(bo.getFxdgrp());
        // 1. Create (rsp. update) Contract (incl. Texts)
        bo.getFxdgrp().getRec().setCnfdat( Dates.today());
        if( MdaUtils.compareTo(bo.getFxdgrp().getCbs().getOpn1().getAmt(),0) == 0 && MdaUtils.compareTo(bo.getCanamt(),bo.getFxdgrp().getCbs().getOpn2().getAmt()) == 0 )
        {
            bo.getFxdgrp().getRec().setClsdat( Dates.today());
        }
        fxToolComponent.sav(bo.getFxdgrp(),Decimals.ZERO_SCALE2);
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract ( trnmodVo);
        cbsSav(bo,trnmodVo);
        // save documents
        //记录116台账
        //todo
        //this.getNosmod().savNostro();
    }

    public void cbsSav(FxtlcnStoreBo transactionVo, TrnmodVo trnmodVo) {
        String trninr = transactionHelpService.getTrninr();
        cbsmodService.cbsTrnSetNew("TRN",trninr);
        trnmodService.cbsTrnStore("OPNAMT","AMT1",Dates.today(),transactionVo.getCancur(),transactionVo.getCanamt().negate(), "Liquidation",transactionVo.getFxdgrp().getRec(),trninr);
        //liaallService.storeAll(transactionHelpService.getTransactionStoreSet().getLiaall());
    }



    public List<String> defaultAcc2N1000(FxtlcnStoreVo vo){
        vo.reset();
        Fxd rec=vo.getFxdgrp().getRec();

        List<String> actLst=new ArrayList<>();
        if( ! MdaUtils.isEmpty(vo.getFxdgrp().getCbs().getMax().getCur()) && ! MdaUtils.isEmpty(vo.getAplacc().getPts().getPtyinr()) )
        {
            String sql = "WHERE CUR = '" + vo.getFxdgrp().getCbs().getMax().getCur() + "' AND SERPTYINR = '" + vo.getAplacc().getPts().getPtyinr() + "' AND TYP = 'DB'";
            List<String > actList = actMapper.getSerAccByConditionSql(sql);

            if( ! vo.isModified("acc2") && actLst.size()>0) {
                vo.setAcc2( actLst.get(1));
            }
        }
        else
        {
            vo.setAcc2(NULLSTR);
        }


        return actLst;
    }

    /**
     *   source:fxtlcn.0009.script
     */

    public  List<String> defaultAcc1N1000(FxtlcnStoreVo vo){
        vo.reset();
        Fxd rec=vo.getFxdgrp().getRec();

        List<String> actLst=new ArrayList<>();
        if( ! MdaUtils.isEmpty(vo.getFxdgrp().getCbs().getMax().getCur()) && ! MdaUtils.isEmpty(vo.getFxdgrp().getAct().getPts().getPtyinr()) )
        {
            String sql = "WHERE CUR = '" + vo.getFxdgrp().getCbs().getMax().getCur() + "' AND SERPTYINR = '" + vo.getFxdgrp().getAct().getPts().getPtyinr() + "' AND TYP = 'DB'";
            List<String > actList = actMapper.getSerAccByConditionSql(sql);

            if( ! vo.isModified("acc1") && actLst.size()>0) {
                vo.setAcc1( actLst.get(1));
            }
        }
        else
        {
            vo.setAcc1(NULLSTR);
        }


        return actLst;
    }
}
