package com.brilliance.isc.funds.fxtsqo.register.settle;

import com.brilliance.isc.bo.Cbb;
import com.brilliance.isc.common.register.settle.DefaultSettleRegister;
import com.brilliance.isc.common.register.settle.ISettleRegister;
import com.brilliance.isc.common.setglg.service.SetglgService;
import com.brilliance.isc.common.setglg.vo.SetglgVo;
import com.brilliance.isc.common.setmod.vo.SetmodVo;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Map;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

@Component("fxtsqo.settleRegister")
public class FxtsqoSettleRegister implements ISettleRegister {

    @Autowired
    private SetglgService setglgService;
    @Autowired
    private DefaultSettleRegister defaultSettleRegister;


    @Override
    public void registerSettlement(SetmodVo setmodVo) {
        transRegister(setmodVo);
        defaultSettleRegister.registerSettlement(setmodVo);
    }

    /**
     * source:fxtsqo.0005.script
     */
    private void transRegister(SetmodVo setmodVo) {
        FxdBizInfoVo rec = (FxdBizInfoVo) setmodVo.getRec();
        Map<String, Cbb> cbsMap = setmodVo.getCbsMap();
        BigDecimal maxAmt = BigDecimal.ZERO;
        if (cbsMap.get("MAX") != null) {
            maxAmt = setmodVo.getCbsMap().get("MAX").getAmt();
        }
        String nom1Cur = NULLSTR;
        if (cbsMap.get("NOM1") != null) {
            nom1Cur = setmodVo.getCbsMap().get("NOM1").getCur();
        }
        setmodVo.setDocamt(maxAmt);
        setmodVo.setOpnamt(maxAmt);
        SetglgVo setglg = setmodVo.getSetglg();
        setglgService.setglgClrAmount(setglg, "TRN");
        setglgService.setglgClrAmount(setglg, "SET");
        String dsp1 = NULLSTR;
        String dsp2 = NULLSTR;
        String acc = NULLSTR;
        String acc2 = NULLSTR;
        String dbtKey = NULLSTR;
        String cdtKey = NULLSTR;
        if (!MdaUtils.isEmpty(rec.getRat())) {
            if (MdaUtils.compareTo(Strings.mid(rec.getFxtyp(), 1, 1), "S") == 0) {
                if (MdaUtils.compareTo(rec.getDsp(), "CA") == 0) {
                    dsp1 = "LO";
                } else {
                    dsp1 = rec.getDsp();
                }
                if (MdaUtils.compareTo(rec.getDsp2(), "CA") == 0) {
                    dsp2 = "LO";
                } else {
                    dsp2 = rec.getDsp2();
                }
                acc = rec.getAcc();
                acc2 = rec.getAcc2();
            } else {
                dsp1 = "RP";
                dsp2 = "RP";
                acc = "";
                acc2 = "";
            }
            if (MdaUtils.compareTo(Strings.mid(rec.getFxtyp(), 2, 1), "B") == 0) {
                dbtKey = setglgService.setglgAddAmount(setglg, "TRN", "APL", setmodVo.getDoccur(), setmodVo.getSetamt(), "", dsp1 + "D", "F", acc, "");
                cdtKey = setglgService.setglgAddAmount(setglg, "SET", "APL", setmodVo.getDoccur(), setmodVo.getSetamt().negate(), "SET", dsp2 + "C", "F", acc2, "SET");
                setglgService.setglgAddAmtFldAccCur(setglg, "SET", cdtKey, nom1Cur);
                setglgService.setglgAddAmtFldExtRat(setglg, "SET", cdtKey, BigDecimal.ONE, rec.getRat());
                setglgService.setSettlementDetails(setglg, "B", "APL", rec.getTrnman(), rec.getTrdint());
            } else {
                cdtKey = setglgService.setglgAddAmount(setglg, "TRN", "APL", setmodVo.getDoccur(), setmodVo.getSetamt().negate(), "", dsp1 + "C", "F", acc, "");
                dbtKey = setglgService.setglgAddAmount(setglg, "SET", "APL", setmodVo.getDoccur(), setmodVo.getSetamt(), "SET", dsp2 + "D", "F", acc2, "SET");
                setglgService.setglgAddAmtFldAccCur(setglg, "SET", dbtKey, nom1Cur);
                setglgService.setglgAddAmtFldExtRat(setglg, "SET", dbtKey, BigDecimal.ONE, rec.getRat());
                setglgService.setSettlementDetails(setglg, "S", "APL", rec.getTrnman(), rec.getTrdout());
            }
        }


    }
}
